#include "test/Service.h"

#include <nlohmann/json.hpp>

#include <memory>


namespace xs { namespace test 
{
    Service::Service(const nlohmann::json& object)
    : Service(
        object.count("command") ? Type::COMMAND : Type::EVENT,
        object[object.count("command") ? "command" : "event"].get<std::string>(),
        object["response"]
    )
    {
        /// TODO: validate the response string here
    }

    bool Service::verify(const nlohmann::json& object) noexcept
    {
        if (!(object.is_object()
              && (object.count("command") || object.count("event"))
              && object.count("response"))) {
            return false;
        }

        const bool isCommand = object.count("command");

        return (
            (
                (isCommand && object["command"].is_string())
                || (!isCommand && object["event"].is_string())
            )
            && (
                object["response"].is_string() || object["response"].is_object()
            )
        );
    }

    Service::Service(Type type, std::string name, nlohmann::json response) noexcept
    : type_(type)
    , name_(std::move(name))
    , response_(std::move(response))
    { }

    bool Service::isCommand() const noexcept
    {
        return type_ == Type::COMMAND;
    }

    bool Service::isEvent() const noexcept
    {
        return type_ == Type::EVENT;
    }
} }
