#include "test/Service.h"
#include "test/Tester.h"

#include <boost/range/adaptors.hpp>

#include <chrono>
#include <functional>
#include <thread>


namespace xs { namespace test {

Tester::Tester(const boost::filesystem::path& file)
: config_(file)
, server_(
    "xs-test", { }, { }, 0, nullptr
)
{
    registerServices();
}

void Tester::registerServices() noexcept
{
    acceptCommands_.clear();
    acceptEvents_.clear();

    using namespace std::placeholders;
    for (const auto& service : config_.getServices()) {
        if (service.isCommand()) {
            server_.onCmd(
                service.name_,
                std::bind(&Tester::onCommand, this, _1, _2)
            );
            acceptCommands_[service.name_] = &service;
        } else {
            server_.onEvent(
                service.name_,
                std::bind(&Tester::onEvent, this, _1, _2)
            );
            acceptEvents_[service.name_] = &service;
        }
    }
} 

std::string Tester::onCommand(const std::string& name, const std::string& params) noexcept
{
    const auto found = acceptCommands_.find(name);
    const auto* service = std::get<1>(*found);
    const auto& response = service->response_;

    /// TODO: keep track the status

    return response.is_string() ? response.get<std::string>()
                                : response.dump();
}

void Tester::onEvent(const std::string& name, const std::string& params) noexcept
{
    const auto found = acceptEvents_.find(name);
    const auto* service = std::get<1>(*found);
    const auto& response = service->response_;

    /// TODO: keep track the status
}

void Tester::wait() noexcept
{
    using namespace std::chrono_literals;
    while (true) {
        std::this_thread::sleep_for(1s);
    }
}

} }
